﻿using AppointmentsWebPart.VisualWebParts;
using Microsoft.QualityTools.Testing.Emulators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Emulators;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AppointmentsWebPart.Tests
{
    class PreBakedSouffle
    {
        [TestMethod]
        public void ScheduleAppointmentReturnsFalseWhenPassedInvalidCorpId()
        {
            using (new SharePointEmulationScope(EmulationMode.Enabled))
            {
                SPSite site = new SPSite("http://localhost");
                string listName = String.Format("List{0}", Guid.NewGuid());

                // create a new temporary list
                Guid listId = site.RootWeb.Lists.Add(listName, listName, SPListTemplateType.GenericList);
                SPList list = site.RootWeb.Lists[listId];
                Assert.IsNotNull(list);

                // add fields to list
                list.Fields.Add("Name", SPFieldType.Text, true);
                list.Fields.Add("Phone", SPFieldType.Text, false);
                list.Fields.Add("Email", SPFieldType.Text, false);
                list.Fields.Add("Age", SPFieldType.Text, false);
                list.Fields.Add("Date", SPFieldType.Text, false);
                list.Update();

                // prepare
                string errorMsg = string.Empty;
                DateTime date = DateTime.Now;
                BookAnAppointmentWebPart webPart =
                    new BookAnAppointmentWebPart();

                // act
                bool success = webPart.ScheduleAppointment(
                    site.RootWeb, list.Title,
                    "My Name", "888-888-8888", "My.Name@contoso.com",
                    "23", date, "aasdf", out errorMsg);

                // assert
                Assert.IsFalse(success);

                // cleanup
                list.Delete();
                site.Dispose();
            }
        }

        [TestMethod]
        public void ScheduleAppointmentReturnsTrueWhenPassedValidCorpId()
        {
            using (new SharePointEmulationScope(EmulationMode.Enabled))
            {
                SPSite site = new SPSite("http://localhost");
                string listName = String.Format("List{0}", Guid.NewGuid());

                // create a new temporary list
                Guid listId = site.RootWeb.Lists.Add(listName, listName, SPListTemplateType.GenericList);
                SPList list = site.RootWeb.Lists[listId];
                Assert.IsNotNull(list);

                // add fields to list
                list.Fields.Add("Name", SPFieldType.Text, true);
                list.Fields.Add("Phone", SPFieldType.Text, false);
                list.Fields.Add("Email", SPFieldType.Text, false);
                list.Fields.Add("Age", SPFieldType.Text, false);
                list.Fields.Add("Date", SPFieldType.Text, false);
                list.Update();

                // prepare
                string errorMsg = string.Empty;
                DateTime date = DateTime.Now;
                BookAnAppointmentWebPart webPart =
                    new BookAnAppointmentWebPart();

                // act
                bool success = webPart.ScheduleAppointment(
                    site.RootWeb, list.Title,
                    "My Name", "888-888-8888", "My.Name@contoso.com",
                    "23", date, "a123456", out errorMsg);

                // assert
                Assert.IsTrue(success);

                // cleanup
                list.Delete();
                site.Dispose();
            }
        }
    }
}
