﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.SharePoint;
using AppointmentsWebPart.VisualWebParts;
using Microsoft.SharePoint.Emulators;
using Microsoft.QualityTools.Testing.Emulators;
using Microsoft.SharePoint.Fakes;
using Microsoft.QualityTools.Testing.Fakes;


namespace AppointmentsWebPart.Tests
{
    [TestClass]
    public class UnitTest1
    {
        [TestMethod]
        public void ScheduleAppointmentReturnsTrueWhenNewAppointmentIsCreated()
        {
            using (new SharePointEmulationScope(EmulationMode.Enabled))
            {
                SPSite site = new SPSite("http://localhost");
                string listName = String.Format("List{0}", Guid.NewGuid());

                // create a new temporary list
                Guid listId = site.RootWeb.Lists.Add(listName, listName, SPListTemplateType.GenericList);
                SPList list = site.RootWeb.Lists[listId];
                Assert.IsNotNull(list);

                // add fields to list
                list.Fields.Add("Name", SPFieldType.Text, true);
                list.Fields.Add("Phone", SPFieldType.Text, false);
                list.Fields.Add("Email", SPFieldType.Text, false);
                list.Fields.Add("Age", SPFieldType.Text, false);
                list.Fields.Add("Date", SPFieldType.Text, false);
                list.Update();

                // prepare
                string errorMsg = string.Empty;
                DateTime date = DateTime.Now;
                BookAnAppointmentWebPart webPart = 
                    new BookAnAppointmentWebPart();

                // act
                bool success = webPart.ScheduleAppointment(
                    site.RootWeb, list.Title, 
                    "My Name", "888-888-8888", "My.Name@contoso.com", 
                    "23", date, out errorMsg);

                // assert
                Assert.IsTrue(success);

                // cleanup
                list.Delete();
                site.Dispose();
            }
        }

        [TestMethod]
        public void GetAppointmentsForTodayReturnsTwoAppointments()
        {
            using (var emulationScope = new SharePointEmulationScope(EmulationMode.Enabled))
            {
                SPSite site = new SPSite("http://localhost");
                string listName = String.Format("List{0}", Guid.NewGuid());

                // create a new temporary list
                Guid listId = site.RootWeb.Lists.Add(listName, listName, SPListTemplateType.GenericList);
                SPList list = site.RootWeb.Lists[listId];
                Assert.IsNotNull(list);

                // add fields to list
                list.Fields.Add("Name", SPFieldType.Text, true);
                list.Fields.Add("Phone", SPFieldType.Text, false);
                list.Fields.Add("Email", SPFieldType.Text, false);
                list.Fields.Add("Age", SPFieldType.Text, false);
                list.Fields.Add("Date", SPFieldType.Text, false);
                list.Update();

                DateTime date = DateTime.Now;
                BookAnAppointmentWebPart webPart = new BookAnAppointmentWebPart();

                // insert 3 item into list
                SPItem item = list.Items.Add();
                item["Name"] = "My Name";
                item["Phone"] = "888-888-8888";
                item["Email"] = "My.Name@contoso.com";
                item["Age"] = "23";
                item["Date"] = date.ToString("D");
                item.Update();

                SPItem item2 = list.Items.Add();
                item2["Name"] = "His Name";
                item2["Phone"] = "777-777-7777";
                item2["Email"] = "His.Name@contoso.com";
                item2["Age"] = "25";
                item2["Date"] = date.AddDays(1).ToString("D");
                item2.Update();

                if (emulationScope.EmulationMode == EmulationMode.Enabled)
                {
                    var sList = new ShimSPList(list);
                    sList.GetItemsSPQuery = (query) =>
                    {
                        var shim = new ShimSPListItemCollection();
                        shim.Bind(new[] { list.Items[0] });
                        return shim.Instance;
                    };
                }

                // Act
                string result = webPart.GetAppointmentsForToday(list.Title, site.RootWeb);

                // Assert
                Assert.IsTrue(result.Contains(String.Format("Name: My Name, Phone: 888-888-8888, Email: My.Name@contoso.com, Age: 23, Date: {0}", date.ToString("D"))));
                Assert.IsFalse(result.Contains("Name: His Name"));

                // cleanup
                list.Delete();
                site.Dispose();
            }
        }
    }
}
