﻿// ********************************************************
// * *
// * Copyright (C) Microsoft. All rights reserved. *
// * *
// ********************************************************

using Microsoft.SharePoint;
using System;
using System.ComponentModel;

namespace AppointmentsWebPart.VisualWebParts
{
    [ToolboxItem(false)]
    public partial class BookAnAppointmentWebPart : System.Web.UI.WebControls.WebParts.WebPart
    {
        // Uncomment the following SecurityPermission attribute only when doing Performance Profiling using
        // the Instrumentation method, and then remove the SecurityPermission attribute when the code is ready
        // for production. Because the SecurityPermission attribute bypasses the security check for callers of
        // your constructor, it's not recommended for production purposes.
        // [System.Security.Permissions.SecurityPermission(System.Security.Permissions.SecurityAction.Assert, UnmanagedCode = true)]
        public BookAnAppointmentWebPart()
        {
        }

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            this.InitializeControl();
        }

        protected void Page_Load(object sender, EventArgs e)
        {
        }

        protected void ScheduleAppointmentButton_Click(object sender, EventArgs e)
        {
            string errorMsg = string.Empty;
            SPWeb web = SPContext.Current.Web;
            try
            {
                bool success = this.ScheduleAppointment(web, "Appointments",
                    this.NameTextBox.Text, this.PhoneTextBox.Text,
                    this.EmailTextBox.Text, this.AgeTextBox.Text,
                    this.AppointmentsCalendar.SelectedDate,
                    this.CorpId.Text, 
                    out errorMsg);

                if (!success)
                    throw new Exception(errorMsg);

                // clear text boxes and labels
                this.ErrorMessageLabel.Text = string.Empty;
                this.NameTextBox.Text = this.PhoneTextBox.Text = this.EmailTextBox.Text = this.AgeTextBox.Text = string.Empty;
            }
            catch (Exception ex)
            {
                this.ErrorMessageLabel.Text = ex.Message;
            }
        }

        public bool ScheduleAppointment(
            SPWeb web, string listName, string name, 
            string phone, string email, string age, 
            DateTime date, string corpId, 
            out string errorMsg)
        {
            if (IsValidCorpId(corpId) == false)
            {
                errorMsg = "invalid corp id";
                return false;
            }
            else
            {
                return ScheduleAppointment(
                    web, listName, name, phone,
                    email, age, date, out errorMsg);
            }
        }

        public bool ScheduleAppointment(SPWeb web, string listName, string name, string phone, string email, string age, DateTime date, out string errorMsg)
        {
            errorMsg = string.Empty;
            var badFormat = this.checkInput(name, phone, email, age);
            if (badFormat)
            {
                errorMsg = "Bad Format";
                return false;
            }
            var exists = this.CheckDuplicate(listName, web, name, phone, email, age, date);
            if (exists)
            {
                errorMsg = "Item already exists";
                return false;
            }

            SPListItemCollection items = web.Lists[listName].Items;

            // create item and populate fields
            SPListItem item = items.Add();
            item["Name"] = name;
            item["Phone"] = phone;
            item["Email"] = email;
            item["Age"] = age;
            item["Date"] = date.ToString("D");

            item.Update();

            return true;
        }

        private bool checkInput(string name, string phone, string email, string age)
        {
            string phonePattern = @"^\d{3}-\d{3}-\d{4}$";
            string emailPattern = @"\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*";

            if (!name.Contains(" ") ||
                !System.Text.RegularExpressions.Regex.IsMatch(phone, phonePattern) ||
                !System.Text.RegularExpressions.Regex.IsMatch(email, emailPattern) ||
                Convert.ToInt32(age) <= 0)
                return true;

            return false;
        }

        public bool CheckDuplicate(string tableName, SPWeb web, string name, string phone, string email, string age, DateTime date)
        {
            SPList list = web.Lists[tableName];

            foreach (SPItem item in list.Items)
            {
                if (item["Name"].ToString().Equals(name) &&
                    item["Phone"].ToString().Equals(phone) &&
                    item["Email"].ToString().Equals(email) &&
                    item["Date"].Equals(date.ToString("D")))
                    return true;
            }

            return false;
        }

        protected void CancelButton_Click(object sender, EventArgs e)
        {
            string errorMsg = string.Empty;

            SPWeb web = SPContext.Current.Web;
            try
            {
                bool success = this.CancelAppointment(web,
                    "Appointments",
                    this.NameTextBox.Text,
                    this.PhoneTextBox.Text,
                    this.EmailTextBox.Text,
                    this.AgeTextBox.Text,
                    this.AppointmentsCalendar.SelectedDate,
                    out errorMsg);

                if (!success)
                    throw new Exception(errorMsg);

                // clear text boxes and labels
                this.ErrorMessageLabel.Text = string.Empty;
                this.NameTextBox.Text = this.PhoneTextBox.Text = this.EmailTextBox.Text = this.AgeTextBox.Text = string.Empty;
            }
            catch (Exception ex)
            {
                this.ErrorMessageLabel.Text = ex.Message;
            }
        }

        public bool CancelAppointment(SPWeb web, string listName, string name, string phone, string email, string age, DateTime date, out string errorMsg)
        {
            errorMsg = string.Empty;

            var badFormat = this.checkInput(name, phone, email, age);
            if (badFormat)
            {
                errorMsg = "Bad Format";
                return false;
            }

            SPItem item = this.findItem(listName, web, name, phone, email, age, date);
            if (item == null)
            {
                errorMsg = "Appointment Not Found";
                return false;
            }

            item.Delete();

            return true;
        }

        private SPItem findItem(string listName, SPWeb web, string name, string phone, string email, string age, DateTime date)
        {
            SPList list = web.Lists[listName];

            foreach (SPItem item in list.Items)
            {
                if (item["Name"].ToString().Equals(name) &&
                    item["Phone"].ToString().Equals(phone) &&
                    item["Email"].ToString().Equals(email) &&
                    item["Age"].ToString().Equals(age) &&
                    item["Date"].Equals(date.ToString("D")))
                    return item;
            }

            return null;
        }

        public string GetAppointmentsForToday(string listName, SPWeb web)
        {
            SPList list = web.Lists[listName];
            DateTime today = DateTime.Now;

            var listQuery = new SPQuery
            {
                Query = String.Format(
                    "<Where>" +
                    "<Eq>" +
                    "<FieldRef Name='Date'/>" +
                    "<Value Type='Text'>{0}</Value>" +
                    "</Eq>" +
                    "</Where>", today.ToString("D"))
            };

            var result = new System.Text.StringBuilder();
            foreach (SPListItem item in list.GetItems(listQuery))
            {
                result.AppendFormat("Name: {0}, Phone: {1}, Email: {2}, Age: {3}, Date: {4}\n", item["Name"], item["Phone"], item["Email"], item["Age"], item["Date"]);
            }

            return result.ToString();
        }

        protected void ValidateCorpId_Click(object sender, EventArgs e)
        {
            var corpIdValue = CorpId.Text;

            if (IsValidCorpId(corpIdValue) == true)
            {
                LabelIsCorpIdValid.Text = "It's GOOD!!!";
            }
            else
            {
                LabelIsCorpIdValid.Text = "Try again, Einstein.";
            }
        }

        public bool IsValidCorpId(string corpIdValue)
        {
            if (String.IsNullOrEmpty(corpIdValue) == true)
            {
                return false;
            }
            else
            {
                if (corpIdValue.Length != 7)
                {
                    return false;
                }
                else
                {
                    if (Char.IsLetter(corpIdValue[0]) == false)
                    {
                        return false;
                    }
                    else
                    {
                        for (int index = 1; index < corpIdValue.Length; index++)
                        {
                            if (Char.IsNumber(corpIdValue[index]) == false)
                            {
                                return false;
                            }
                        }
                    }

                    return true;
                }
            }
        }
    }
}
